@@ -1,14 +1,16 @@
 from __future__ import absolute_import
 import copy
 import hashlib
+import locale
 import json
 import logging
 import os
 import re
 import shutil
 import tempfile
-from functools import partial
-from typing import Any, Callable, Dict, Generator, List, Optional, Set, Text, Union, cast
+from functools import partial, cmp_to_key
+from typing import (Any, Callable, Dict, Generator, List, Optional, Set, Text,
+    Union, cast)
 
 from six import string_types, u
 
@@ -119,38 +121,35 @@ def revmap_file(builder, outdir, f):
     # builder.outdir is the inner (container/compute node) output directory
     # outdir is the outer (host/storage system) output directory
 
-    if "location" in f:
+    if "location" in f and "path" not in f:
         if f["location"].startswith("file://"):
-            path = convert_pathsep_to_unix(uri_file_path(f["location"]))
-            revmap_f = builder.pathmapper.reversemap(path)
-
-            if revmap_f and not builder.pathmapper.mapper(revmap_f[0]).type.startswith("Writable"):
-                f["basename"] = os.path.basename(path)
-                f["location"] = revmap_f[1]
-            elif path == builder.outdir:
-                f["location"] = outdir
-            elif path.startswith(builder.outdir):
-                f["location"] = builder.fs_access.join(outdir, path[len(builder.outdir) + 1:])
-            elif f["location"].startswith(outdir):
-                revmap_f = builder.pathmapper.reversemap(builder.fs_access.join(builder.outdir, f["location"][len(outdir) + 1:]))
-                if revmap_f and not builder.pathmapper.mapper(revmap_f[0]).type.startswith("Writable"):
-                    f["basename"] = os.path.basename(path)
-                    f["location"] = revmap_f[1]
-        return f
+            f["path"] = convert_pathsep_to_unix(uri_file_path(f["location"]))
+        else:
+            return f
 
     if "path" in f:
         path = f["path"]
+        uripath = file_uri(path)
         del f["path"]
+
+        if "basename" not in f:
+            f["basename"] = os.path.basename(path)
+
         revmap_f = builder.pathmapper.reversemap(path)
-        if revmap_f:
+
+        if revmap_f and not builder.pathmapper.mapper(revmap_f[0]).type.startswith("Writable"):
             f["location"] = revmap_f[1]
-            return f
-        elif path.startswith(builder.outdir):
+        elif uripath == outdir or uripath.startswith(outdir+os.sep):
+            f["location"] = file_uri(path)
+        elif path == builder.outdir or path.startswith(builder.outdir+os.sep):
             f["location"] = builder.fs_access.join(outdir, path[len(builder.outdir) + 1:])
-            return f
+        elif not os.path.isabs(path):
+            f["location"] = builder.fs_access.join(outdir, path)
         else:
             raise WorkflowException(u"Output file path %s must be within designated output directory (%s) or an input "
                                     u"file pass through." % (path, builder.outdir))
+        return f
+
 
     raise WorkflowException(u"Output File object is missing both `location` and `path` fields: %s" % f)
 
@@ -211,7 +210,7 @@ class CommandLineTool(Process):
                     })
                     dockerReq = self.requirements[0]
                     if default_container == windows_default_container_id and use_container and onWindows():
-                        _logger.warning(DEFAULT_CONTAINER_MSG%(windows_default_container_id, windows_default_container_id))
+                        _logger.warning(DEFAULT_CONTAINER_MSG % (windows_default_container_id, windows_default_container_id))
 
         if dockerReq and use_container:
             return DockerCommandLineJob()
@@ -247,7 +246,6 @@ class CommandLineTool(Process):
         # type: (...) -> Generator[Union[JobBase, CallbackJob], None, None]
 
         jobname = uniquename(kwargs.get("name", shortname(self.tool.get("id", "job"))))
-
         if kwargs.get("cachedir"):
             cacheargs = kwargs.copy()
             cacheargs["outdir"] = "/out"
@@ -268,11 +266,15 @@ class CommandLineTool(Process):
                 dockerimg = docker_req.get("dockerImageId") or docker_req.get("dockerPull")
             elif kwargs.get("default_container", None) is not None and kwargs.get("use_container"):
                 dockerimg = kwargs.get("default_container")
+            else:
+                dockerimg = None
 
             if dockerimg:
                 cmdline = ["docker", "run", dockerimg] + cmdline
             keydict = {u"cmdline": cmdline}
 
+            if "stdout" in self.tool:
+                keydict["stdout"] = self.tool["stdout"]
             for location, f in cachebuilder.pathmapper.items():
                 if f.type == "File":
                     checksum = next((e['checksum'] for e in cachebuilder.files
@@ -349,7 +351,9 @@ class CommandLineTool(Process):
         j.hints = self.hints
         j.name = jobname
 
-        if _logger.isEnabledFor(logging.DEBUG):
+        debug = _logger.isEnabledFor(logging.DEBUG)
+
+        if debug:
             _logger.debug(u"[job %s] initializing from %s%s",
                           j.name,
                           self.tool.get("id", ""),
@@ -408,28 +412,28 @@ class CommandLineTool(Process):
                 self.updatePathmap(builder.outdir, builder.pathmapper, l)
             visit_class([builder.files, builder.bindings], ("File", "Directory"), _check_adjust)
 
-        if _logger.isEnabledFor(logging.DEBUG):
+        if debug:
             _logger.debug(u"[job %s] path mappings is %s", j.name,
                           json.dumps({p: builder.pathmapper.mapper(p) for p in builder.pathmapper.files()}, indent=4))
 
         if self.tool.get("stdin"):
-            with SourceLine(self.tool, "stdin", validate.ValidationException):
+            with SourceLine(self.tool, "stdin", validate.ValidationException, debug):
                 j.stdin = builder.do_eval(self.tool["stdin"])
                 reffiles.append({"class": "File", "path": j.stdin})
 
         if self.tool.get("stderr"):
-            with SourceLine(self.tool, "stderr", validate.ValidationException):
+            with SourceLine(self.tool, "stderr", validate.ValidationException, debug):
                 j.stderr = builder.do_eval(self.tool["stderr"])
                 if os.path.isabs(j.stderr) or ".." in j.stderr:
                     raise validate.ValidationException("stderr must be a relative path, got '%s'" % j.stderr)
 
         if self.tool.get("stdout"):
-            with SourceLine(self.tool, "stdout", validate.ValidationException):
+            with SourceLine(self.tool, "stdout", validate.ValidationException, debug):
                 j.stdout = builder.do_eval(self.tool["stdout"])
                 if os.path.isabs(j.stdout) or ".." in j.stdout or not j.stdout:
                     raise validate.ValidationException("stdout must be a relative path, got '%s'" % j.stdout)
 
-        if _logger.isEnabledFor(logging.DEBUG):
+        if debug:
             _logger.debug(u"[job %s] command line bindings is %s", j.name, json.dumps(builder.bindings, indent=4))
 
         dockerReq = self.get_requirement("DockerRequirement")[0]
@@ -508,37 +512,31 @@ class CommandLineTool(Process):
     def collect_output_ports(self, ports, builder, outdir, compute_checksum=True, jobname="", readers=None):
         # type: (Set[Dict[Text, Any]], Builder, Text, bool, Text, Dict[Text, Any]) -> Dict[Text, Union[Text, List[Any], Dict[Text, Any]]]
         ret = {}  # type: Dict[Text, Union[Text, List[Any], Dict[Text, Any]]]
+        debug = _logger.isEnabledFor(logging.DEBUG)
         try:
             fs_access = builder.make_fs_access(outdir)
             custom_output = fs_access.join(outdir, "cwl.output.json")
             if fs_access.exists(custom_output):
                 with fs_access.open(custom_output, "r") as f:
                     ret = json.load(f)
-                if _logger.isEnabledFor(logging.DEBUG):
+                if debug:
                     _logger.debug(u"Raw output from %s: %s", custom_output, json.dumps(ret, indent=4))
             else:
                 for i, port in enumerate(ports):
-                    with SourceLine(ports, i, WorkflowException):
+                    def makeWorkflowException(msg):
+                        return WorkflowException(
+                            u"Error collecting output for parameter '%s':\n%s"
+                            % (shortname(port["id"]), msg))
+                    with SourceLine(ports, i, makeWorkflowException, debug):
                         fragment = shortname(port["id"])
-                        try:
-                            ret[fragment] = self.collect_output(port, builder, outdir, fs_access,
-                                                                compute_checksum=compute_checksum)
-                        except Exception as e:
-                            _logger.debug(
-                                u"Error collecting output for parameter '%s'"
-                                % shortname(port["id"]), exc_info=True)
-                            raise WorkflowException(
-                                u"Error collecting output for parameter '%s':\n%s"
-                                % (shortname(port["id"]), indent(u(str(e)))))
-
+                        ret[fragment] = self.collect_output(port, builder, outdir, fs_access,
+                                                            compute_checksum=compute_checksum)
             if ret:
                 revmap = partial(revmap_file, builder, outdir)
                 adjustDirObjs(ret, trim_listing)
                 visit_class(ret, ("File", "Directory"), cast(Callable[[Any], Any], revmap))
                 visit_class(ret, ("File", "Directory"), remove_path)
                 normalizeFilesDirs(ret)
-                if builder.mutation_manager:
-                    adjustFileObjs(ret, builder.mutation_manager.set_generation)
                 visit_class(ret, ("File", "Directory"), partial(check_valid_locations, fs_access))
 
                 if compute_checksum:
@@ -546,6 +544,8 @@ class CommandLineTool(Process):
 
             validate.validate_ex(self.names.get_name("outputs_record_schema", ""), ret,
                                  strict=False, logger=_logger_validation_warnings)
+            if ret is not None and builder.mutation_manager is not None:
+                adjustFileObjs(ret, builder.mutation_manager.set_generation)
             return ret if ret is not None else {}
         except validate.ValidationException as e:
             raise WorkflowException("Error validating output record. " + Text(e) + "\n in " + json.dumps(ret, indent=4))
@@ -557,6 +557,7 @@ class CommandLineTool(Process):
     def collect_output(self, schema, builder, outdir, fs_access, compute_checksum=True):
         # type: (Dict[Text, Any], Builder, Text, StdFsAccess, bool) -> Union[Dict[Text, Any], List[Union[Dict[Text, Any], Text]]]
         r = []  # type: List[Any]
+        debug = _logger.isEnabledFor(logging.DEBUG)
         if "outputBinding" in schema:
             binding = schema["outputBinding"]
             globpatterns = []  # type: List[Text]
@@ -564,7 +565,7 @@ class CommandLineTool(Process):
             revmap = partial(revmap_file, builder, outdir)
 
             if "glob" in binding:
-                with SourceLine(binding, "glob", WorkflowException):
+                with SourceLine(binding, "glob", WorkflowException, debug):
                     for gb in aslist(binding["glob"]):
                         gb = builder.do_eval(gb)
                         if gb:
@@ -576,16 +577,25 @@ class CommandLineTool(Process):
                         elif gb == ".":
                             gb = outdir
                         elif gb.startswith("/"):
-                            raise WorkflowException("glob patterns must not start with '/'")
+                            raise WorkflowException(
+                                "glob patterns must not start with '/'")
                         try:
                             prefix = fs_access.glob(outdir)
                             r.extend([{"location": g,
-                                       "path": fs_access.join(builder.outdir, g[len(prefix[0])+1:]),
+                                       "path": fs_access.join(builder.outdir,
+                                           g[len(prefix[0])+1:]),
                                        "basename": os.path.basename(g),
-                                       "nameroot": os.path.splitext(os.path.basename(g))[0],
-                                       "nameext": os.path.splitext(os.path.basename(g))[1],
-                                       "class": "File" if fs_access.isfile(g) else "Directory"}
-                                      for g in fs_access.glob(fs_access.join(outdir, gb))])
+                                       "nameroot": os.path.splitext(
+                                           os.path.basename(g))[0],
+                                       "nameext": os.path.splitext(
+                                           os.path.basename(g))[1],
+                                       "class": "File" if fs_access.isfile(g)
+                                       else "Directory"}
+                                      for g in sorted(fs_access.glob(
+                                          fs_access.join(outdir, gb)),
+                                          key=cmp_to_key(cast(
+                                              Callable[[Text, Text],
+                                                  int], locale.strcoll)))])
                         except (OSError, IOError) as e:
                             _logger.warning(Text(e))
                         except:
@@ -629,12 +639,12 @@ class CommandLineTool(Process):
                 single = True
 
             if "outputEval" in binding:
-                with SourceLine(binding, "outputEval", WorkflowException):
+                with SourceLine(binding, "outputEval", WorkflowException, debug):
                     r = builder.do_eval(binding["outputEval"], context=r)
 
             if single:
                 if not r and not optional:
-                    with SourceLine(binding, "glob", WorkflowException):
+                    with SourceLine(binding, "glob", WorkflowException, debug):
                         raise WorkflowException("Did not find output file with glob pattern: '{}'".format(globpatterns))
                 elif not r and optional:
                     pass
@@ -645,7 +655,7 @@ class CommandLineTool(Process):
                         r = r[0]
 
             if "secondaryFiles" in schema:
-                with SourceLine(schema, "secondaryFiles", WorkflowException):
+                with SourceLine(schema, "secondaryFiles", WorkflowException, debug):
                     for primary in aslist(r):
                         if isinstance(primary, dict):
                             primary.setdefault("secondaryFiles", [])
