@@ -153,8 +153,8 @@ def next_seg(remain, obj):  # type: (Text, Any) -> Any
         return obj
 
 
-def evaluator(ex, jslib, obj, fullJS=False, timeout=None, force_docker_pull=False, debug=False):
-    # type: (Text, Text, Dict[Text, Any], bool, int, bool, bool) -> JSON
+def evaluator(ex, jslib, obj, fullJS=False, timeout=None, force_docker_pull=False, debug=False, js_console=False):
+    # type: (Text, Text, Dict[Text, Any], bool, int, bool, bool, bool) -> JSON
     m = param_re.match(ex)
     if m:
         if m.end(1)+1 == len(ex) and m.group(1) == "null":
@@ -164,7 +164,7 @@ def evaluator(ex, jslib, obj, fullJS=False, timeout=None, force_docker_pull=Fals
         except Exception as w:
             raise WorkflowException("%s%s" % (m.group(1), w))
     elif fullJS:
-        return sandboxjs.execjs(ex, jslib, timeout=timeout, force_docker_pull=force_docker_pull, debug=debug)
+        return sandboxjs.execjs(ex, jslib, timeout=timeout, force_docker_pull=force_docker_pull, debug=debug, js_console=js_console)
     else:
         raise sandboxjs.JavascriptException(
             "Syntax error in parameter reference '%s' or used Javascript code without specifying InlineJavascriptRequirement.",
@@ -172,9 +172,9 @@ def evaluator(ex, jslib, obj, fullJS=False, timeout=None, force_docker_pull=Fals
 
 
 def interpolate(scan, rootvars,
-                timeout=None, fullJS=None, jslib="",force_docker_pull=False,
-                debug=False):
-    # type: (Text, Dict[Text, Any], int, bool, Union[str, Text], bool, bool) -> JSON
+                timeout=None, fullJS=None, jslib="", force_docker_pull=False,
+                debug=False, js_console=False):
+    # type: (Text, Dict[Text, Any], int, bool, Union[str, Text], bool, bool, bool) -> JSON
     scan = scan.strip()
     parts = []
     w = scanner(scan)
@@ -184,7 +184,7 @@ def interpolate(scan, rootvars,
         if scan[w[0]] == '$':
             e = evaluator(scan[w[0] + 1:w[1]], jslib, rootvars, fullJS=fullJS,
                           timeout=timeout, force_docker_pull=force_docker_pull,
-                          debug=debug)
+                          debug=debug, js_console=js_console)
             if w[0] == 0 and w[1] == len(scan):
                 return e
             leaf = json.dumps(e, sort_keys=True)
@@ -202,8 +202,8 @@ def interpolate(scan, rootvars,
 
 
 def do_eval(ex, jobinput, requirements, outdir, tmpdir, resources,
-            context=None, pull_image=True, timeout=None, force_docker_pull=False, debug=False):
-    # type: (Union[dict, AnyStr], Dict[Text, Union[Dict, List, Text]], List[Dict[Text, Any]], Text, Text, Dict[Text, Union[int, Text]], Any, bool, int, bool, bool) -> Any
+            context=None, pull_image=True, timeout=None, force_docker_pull=False, debug=False, js_console=False):
+    # type: (Union[dict, AnyStr], Dict[Text, Union[Dict, List, Text]], List[Dict[Text, Any]], Text, Text, Dict[Text, Union[int, Text]], Any, bool, int, bool, bool, bool) -> Any
 
     runtime = copy.copy(resources)
     runtime["tmpdir"] = docker_windows_path_adjust(tmpdir)
@@ -214,7 +214,7 @@ def do_eval(ex, jobinput, requirements, outdir, tmpdir, resources,
         u"self": context,
         u"runtime": runtime}
 
-    if isinstance(ex, (str, Text)):
+    if isinstance(ex, (str, Text)) and ("$(" in ex or "${" in ex):
         fullJS = False
         jslib = u""
         for r in reversed(requirements):
@@ -230,7 +230,9 @@ def do_eval(ex, jobinput, requirements, outdir, tmpdir, resources,
                                fullJS=fullJS,
                                jslib=jslib,
                                force_docker_pull=force_docker_pull,
-                               debug=debug)
+                               debug=debug,
+                               js_console=js_console)
+
         except Exception as e:
             raise WorkflowException("Expression evaluation error:\n%s" % e)
     else:
